﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace GearHeadSaveDataViewer
{
	public sealed class Character : ListViewItem
	{
		private static readonly string[] jpSkillNames = new string[] { string.Empty, "メック小火器", "メック重火器", "メック白兵戦", "メック格闘技", "メック操縦", "小火器", "重火器", "白兵戦", "武術", "回避", "洞察", "先制", "生命力", "サバイバル", "メック修理", "医療", "電子戦", "急所狙い", "話術", "応急手当", "買い物", "生物工学", "修理", "サイバーテック", "隠密", "持久力", "誘惑", "脅迫", "科学", "集中力", "メック工学", "コード破り", "重量挙げ", "神秘学", "パフォーマンス", "耐性", "調査", "ロボット工学", "リーダーシップ", "飼い馴らし", "すり" };
		private static readonly string[] jpTalentNames = new string[] { string.Empty, "揺るぎない心", "ボディビルダー", "科学的思考", "演出", "カンフー", "合気道", "解剖技術", "鋼の肉体", "スタント運転技術", "博学", "中性的美貌", "駆け引き", "天性の飛行士", "歩行バランス", "走り屋", "戦場のハイエナ", "理想人類の血統", "商才", "調教師", "何でも屋", "衛生兵", "八つ当たり", "忍術", "隠れ撃ち", "壊し屋", "エクストロピアン", "サイバー適合体", "スナイパー" };
		private static readonly int[] emptyAdvancedStatus = new int[8];
		private readonly string name;
		private readonly string job;
		private readonly string mecha;
		private readonly bool isFemale;
		private readonly int scale;
		private readonly int totalExp;
		private readonly int freeExp;
		private readonly int money;
		private readonly int reflexes;
		private readonly int body;
		private readonly int speed;
		private readonly int perception;
		private readonly int craft;
		private readonly int ego;
		private readonly int knowledge;
		private readonly int charm;
		private readonly int heroic;
		private readonly int lawful;
		private readonly int sociable;
		private readonly int easygoing;
		private readonly int cheerful;
		private readonly int renowned;
		private readonly int pragmatic;
		private readonly int[] talents;
		private readonly int[] advancedStatus;
		private readonly SortedDictionary<Skill, int> skillDic;

		public Character(string name, string job, string mecha, int scale, int totalExp, int usedExp, int money, bool isFemale, int reflexes, int body, int speed, int perception, int craft, int ego, int knowledge, int charm, int heroic, int lawful, int sociable, int easygoing, int cheerful, int renowned, int pragmatic, int[] talents, string[] skillLines, string[] advancedStatusLines)
			: base(new string[] { name, job, mecha })
		{
			this.name = name;
			this.job = job;
			this.mecha = mecha;
			this.scale = scale;
			this.totalExp = totalExp;
			this.freeExp = totalExp - usedExp;
			this.money = money;
			this.isFemale = isFemale;
			this.reflexes = reflexes;
			this.body = body;
			this.speed = speed;
			this.perception = perception;
			this.craft = craft;
			this.ego = ego;
			this.knowledge = knowledge;
			this.charm = charm;
			this.heroic = heroic;
			this.lawful = lawful;
			this.sociable = sociable;
			this.easygoing = easygoing;
			this.cheerful = cheerful;
			this.renowned = renowned;
			this.pragmatic = pragmatic;
			this.talents = talents;
			this.skillDic = new SortedDictionary<Skill, int>();
			if (skillLines != null)
			{
				for (int i = 0; i < skillLines.Length; ++i)
				{
					string[] blocks = skillLines[i].Split(' ');
					int skill;
					if (!int.TryParse(blocks[2], out skill) || skill < (int)Skill.Min || skill > (int)Skill.Max)
					{
						continue;
					}
					int value;
					if (!int.TryParse(blocks[3], out value) || value <= 0)
					{
						continue;
					}
					this.skillDic.Add((Skill)skill, value);
				}
			}
			if (advancedStatusLines != null)
			{
				this.advancedStatus = new int[8];
				for (int i = 0; i < advancedStatusLines.Length; ++i)
				{
					string[] blocks = advancedStatusLines[i].Split(' ');
					int ability;
					if (!int.TryParse(blocks[2], out ability) || ability < 1 || ability > 8)
					{
						continue;
					}
					int value;
					if (!int.TryParse(blocks[3], out value) || value <= 0)
					{
						continue;
					}
					this.advancedStatus[--ability] = value;
				}
			}
			else
			{
				this.advancedStatus = emptyAdvancedStatus;
			}
		}

		public void DrawStatus(System.Windows.Forms.RichTextBox rtb)
		{
			const string c = ", ", f = "{0, 3}", f10 = "{0, 10}", f2 = "{0, 2}", t = "\t", t2 = "\t\t", l = "(", r = ")", rt = ")\t";	// comma, format, tab, left, right
			StringBuilder sb = new StringBuilder(8192);
			sb.Append("名前\t");
			sb.Append(this.name);
			sb.Append(Environment.NewLine);
			sb.Append("職業\t");
			sb.Append(this.job);
			sb.Append(Environment.NewLine);
			sb.Append("メック\t");
			sb.Append(this.mecha);
			sb.Append(Environment.NewLine);
			sb.Append("性別\t");
			sb.Append(this.isFemale ? "女性\t" : "男性\t");
			sb.Append("SF ");
			sb.Append(this.scale);
			sb.Append(Environment.NewLine);
			sb.Append("Total Exp\t");
			sb.Append(string.Format(f10, this.totalExp));
			sb.Append(Environment.NewLine);
			sb.Append("Free Exp\t");
			sb.Append(string.Format(f10, this.freeExp));
			sb.Append(Environment.NewLine);
			sb.Append("所持金\t\t");
			sb.Append(string.Format(f10, this.money));
			sb.Append(Environment.NewLine);
			sb.Append("反応\t\t");
			sb.Append(string.Format(f, this.reflexes));
			sb.Append(l);
			sb.Append(string.Format(f2, this.advancedStatus[0]));
			sb.Append(rt);
			sb.Append("工作\t\t");
			sb.Append(string.Format(f, this.craft));
			sb.Append(l);
			sb.Append(string.Format(f2, this.advancedStatus[4]));
			sb.Append(r);
			sb.Append(Environment.NewLine);
			sb.Append("体格\t\t");
			sb.Append(string.Format(f, this.body));
			sb.Append(l);
			sb.Append(string.Format(f2, this.advancedStatus[1]));
			sb.Append(rt);
			sb.Append("自我\t\t");
			sb.Append(string.Format(f, this.ego));
			sb.Append(l);
			sb.Append(string.Format(f2, this.advancedStatus[5]));
			sb.Append(r);
			sb.Append(Environment.NewLine);
			sb.Append("速度\t\t");
			sb.Append(string.Format(f, this.speed));
			sb.Append(l);
			sb.Append(string.Format(f2, this.advancedStatus[2]));
			sb.Append(rt);
			sb.Append("知識\t\t");
			sb.Append(string.Format(f, this.knowledge));
			sb.Append(l);
			sb.Append(string.Format(f2, this.advancedStatus[6]));
			sb.Append(r);
			sb.Append(Environment.NewLine);
			sb.Append("感覚\t\t");
			sb.Append(string.Format(f, this.perception));
			sb.Append(l);
			sb.Append(string.Format(f2, this.advancedStatus[3]));
			sb.Append(rt);
			sb.Append("魅力\t\t");
			sb.Append(string.Format(f, this.charm));
			sb.Append(l);
			sb.Append(string.Format(f2, this.advancedStatus[7]));
			sb.Append(r);
			for (Skill i = Skill.Min; i <= Skill.Max; ++i)
			{
				sb.Append(((int)i & 1) == 0 ? t : Environment.NewLine);
				sb.Append(jpSkillNames[(int)i]);
				sb.Append(jpSkillNames[(int)i].Length > 3 ? t : t2);
				sb.Append(this.skillDic.ContainsKey(i) ? string.Format(f, this.skillDic[i]) : "---");
			}
			sb.Append(Environment.NewLine);
			int cnt = 0;
			if (this.heroic > 0)
			{
				++cnt;
				sb.Append("英雄 ");
				sb.Append(this.heroic);
				sb.Append(c);
			}
			else if (this.heroic < 0)
			{
				++cnt;
				sb.Append("悪辣 ");
				sb.Append(-this.heroic);
				sb.Append(c);
			}
			if (this.lawful > 0)
			{
				++cnt;
				sb.Append("遵法 ");
				sb.Append(this.lawful);
				sb.Append(c);
			}
			else if (this.lawful < 0)
			{
				++cnt;
				sb.Append("無法 ");
				sb.Append(-this.lawful);
				sb.Append(c);
			}
			if (this.sociable > 0)
			{
				++cnt;
				sb.Append("外交的 ");
				sb.Append(this.sociable);
				sb.Append(c);
			}
			else if (this.sociable < 0)
			{
				++cnt;
				sb.Append("内向的 ");
				sb.Append(-this.sociable);
				sb.Append(c);
			}
			if (this.easygoing > 0)
			{
				++cnt;
				sb.Append("呑気 ");
				sb.Append(this.easygoing);
				sb.Append(c);
			}
			else if (this.easygoing < 0)
			{
				++cnt;
				sb.Append("短気 ");
				sb.Append(-this.easygoing);
				sb.Append(c);
			}
			if (this.cheerful > 0)
			{
				if (++cnt == 5)
				{
					sb.Append(Environment.NewLine);
				}
				sb.Append("明朗 ");
				sb.Append(this.cheerful);
				sb.Append(c);
			}
			else if (this.cheerful < 0)
			{
				if (++cnt == 5)
				{
					sb.Append(Environment.NewLine);
				}
				sb.Append("陰鬱 ");
				sb.Append(-this.cheerful);
				sb.Append(c);
			}
			if (this.renowned > 0)
			{
				if (++cnt == 5)
				{
					sb.Append(Environment.NewLine);
				}
				sb.Append("有名 ");
				sb.Append(this.renowned);
				sb.Append(c);
			}
			else if (this.renowned < 0)
			{
				if (++cnt == 5)
				{
					sb.Append(Environment.NewLine);
				}
				sb.Append("負け犬 ");
				sb.Append(-this.renowned);
				sb.Append(c);
			}
			if (this.pragmatic > 0)
			{
				if (++cnt == 5)
				{
					sb.Append(Environment.NewLine);
				}
				sb.Append("実用主義的 ");
				sb.Append(this.pragmatic);
				sb.Append(c);
			}
			else if (this.pragmatic < 0)
			{
				if (++cnt == 5)
				{
					sb.Append(Environment.NewLine);
				}
				sb.Append("神秘主義的 ");
				sb.Append(-this.pragmatic);
				sb.Append(c);
			}
			if (cnt > 0)
			{
				sb.Remove(sb.Length - c.Length, c.Length);
			}
			if (this.talents != null)
			{
				if (cnt > 0)
				{
					sb.Append(Environment.NewLine);
				}
				for (int i = 0; i < this.talents.Length; ++i)
				{
					if (i == 3)
					{
						sb.Append(Environment.NewLine);
					}
					sb.Append(jpTalentNames[this.talents[i]]);
					sb.Append(c);
				}
				sb.Remove(sb.Length - c.Length, c.Length);
			}
			else if (sb.ToString().EndsWith(Environment.NewLine))
			{
				sb.Remove(sb.Length - Environment.NewLine.Length, Environment.NewLine.Length);
			}
			rtb.AppendText(sb.ToString());
		}
	}
}
