﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace GearHeadSaveDataViewer
{
	static class Program
	{
		private static readonly SortedList<string, string> jpJobNameSList = new SortedList<string, string>()
		{
			{ "ARCHITECT", "建築士" }, { "FIREFIGHTER", "消防士" }, { "CONSTRUCTION WORKER", "建設労働者" }, { "JOURNALIST", "報道記者" }, { "BANKER", "銀行員" }, { "LAWYER", "弁護士" }, { "POLITICIAN", "政治家" }, { "EXPLORER", "探検家" }, { "BOUNTY HUNTER", "賞金稼ぎ" }, { "MERCENARY", "傭兵" }, { "CYBERDOC", "サイバー技師" }, { "GUARDIAN", "GUARDIAN隊員" }, { "BANDIT", "山賊" }, { "RAVAGER", "略奪者" }, { "PIRATE", "海賊" }, { "THIEF", "泥棒" }, { "MARTIAL ARTIST", "武術家" }, { "MONK", "モンク僧" }, { "CORPORATE EXECUTIVE", "会社役員" }, { "CORPORATE PILOT", "企業パイロット" }, { "CELEBRITY", "有名人" }, { "MECHANIC", "メカニック" }, { "SHOPKEEPER", "小売店主" }, { "COMBAT PILOT", "戦闘パイロット" }, { "ARENA PILOT", "アリーナパイロット" }, { "RECON PILOT", "偵察パイロット" }, { "PARAMEDIC", "衛生兵" }, { "COMMERCIAL PILOT", "契約パイロット" }, { "ATHLETE", "運動選手" }, { "SOLDIER", "兵士" }, { "NINJA", "ニンジャ" }, { "DOCTOR", "医師" }, { "NURSE", "看護士" }, { "TEST PILOT", "テストパイロット" }, { "TRUCKER", "運び屋" }, { "SCIENTIST", "科学者" }, { "PROFESSOR", "教授" }, { "MECHA DESIGNER", "メック設計者" }, { "ENGINEER", "技術者" }, { "ACTOR", "俳優" }, { "TEACHER", "教師" }, { "COMPUTER PROGRAMMER", "プログラマ" }, { "HACKER", "ハッカー" }, { "ACE PILOT", "エースパイロット" }, { "AMBASSADOR", "大使" }, { "ANIMAL", "動物" }, { "ARCHAEOLOGIST", "考古学者" }, { "ARENA MANAGER", "アリーナ支配人" }, { "ARTIST", "芸術家" }, { "BARTENDER", "バーテンダー" }, { "COOK", "料理人" }, { "CAMPUS SECURITY", "キャンパスの警備員" }, { "CHIEFTAN", "首領" }, { "CITIZEN", "市民" }, { "CORPORATE PRESIDENT", "会社社長" }, { "CURATOR", "館長" }, { "CYCLOPS EXPLORER", "サイクロプスの探索者" }, { "CYCLOPS GUARD", "サイクロプス警備員" }, { "CYCLOPS PROTECTOR", "サイクロプスを守る者" }, { "DEALER", "商人" }, { "ELDER", "長老" }, { "EXECUTIVE ASSISTANT", "重役秘書" }, { "FARMER", "農家" }, { "FISHERMAN", "漁師" }, { "GARU", "GARU" }, { "GANGER", "ギャング" }, { "GRAD STUDENT", "大学院生" }, { "GUARDIAN CHIEF", "GUARDIAN司令官" }, { "GUARDIAN COMMANDER", "GUARDIAN指揮官" }, { "GUILDMASTER", "ギルド長" }, { "HARVESTER", "臓器密売人" }, { "HERBALIST", "植物学者" }, { "HERMIT", "隠者" }, { "HOSTESS", "ホステス" }, { "KEEPER OF THE LAW", "法の番人" }, { "LEADER", "指導者" }, { "MECHA PILOT", "メックパイロット" }, { "MANAGER", "部長" }, { "MONK LEADER", "モンク僧の指導者" }, { "MONK LIBRARIAN", "モンク僧の司書" }, { "PRIEST", "司祭" }, { "PRESIDENT", "社長" }, { "PRINCESS", "王女" }, { "ROBOT", "ロボット" }, { "REFORMED PIRATE", "元海賊" }, { "SECURITY GUARD", "警備員" }, { "SHAMAN", "祈祷師" }, { "LITTLE SISTER", "妹" }, { "SLAVE TRADER", "奴隷商人" }, { "STUDENT", "学生" }, { "TALENT AGENT", "芸能プロダクション" }, { "TANTRIC MECHANIC", "仏教派メカニック" }, { "TICKET VENDOR", "改札員" }, { "TRAPPER", "罠師" }, { "TRUCK DISPATCHER", "運輸配送係" }, { "VILLAGER", "村民" }, { "WARLORD", "軍政長官" }, { "WARRIOR MONK", "モンク僧の武人" }, { "WARRIOR", "武人" }, { "SISTER", "妹" },
		};

		public static string SavePath { get; set; }

		/// <summary>
		/// アプリケーションのメイン エントリ ポイントです。
		/// </summary>
		[STAThread]
		static void Main()
		{
			using (Mutex mutex = new Mutex(false, Application.ProductName))
			{
				if (mutex.WaitOne(0, false))
				{
					string exePath = Application.StartupPath;
#if DEBUG
					exePath = Directory.GetParent(Directory.GetParent(exePath).FullName).FullName;
#endif
					string ini = exePath + Path.DirectorySeparatorChar + "GHSDV.ini", path = null;
					if (File.Exists(ini))
					{
						string text = File.ReadAllText(ini, Encoding.UTF8);
						if (Directory.Exists(text))
						{
							path = text;
						}
					}
					SavePath = path == null ? exePath : path;
					Application.EnableVisualStyles();
					Application.SetCompatibleTextRenderingDefault(false);
					Application.Run(new Form1());
					if (path != SavePath)
					{
						File.WriteAllText(ini, SavePath, Encoding.UTF8);
					}
				}
			}
		}

		public static string ConvertToJapaneseJobName(string job)
		{
			if (!string.IsNullOrEmpty(job))
			{
				string upperJob = job.ToUpperInvariant();
				return jpJobNameSList.ContainsKey(upperJob) ? jpJobNameSList[upperJob] : job;
			}
			else
			{
				return job;
			}
		}
	}
}
